Package: BayesSUR
Type: Package
Title: Bayesian Seemingly Unrelated Regression Models in
        High-Dimensional Settings
Version: 2.3-3
URL: https://github.com/mbant/BayesSUR
BugReports: https://github.com/mbant/BayesSUR/issues
Authors@R: c(person("Marco", "Banterle", email = "marco.banterle@gmail.com", role = c("aut")),
             person("Zhi", "Zhao", email = "zhi.zhao@medisin.uio.no", role = c("aut", "cre")),
             person("Leonardo", "Bottolo", email = "lb664@cam.ac.uk", role = c("ctb")),
             person("Sylvia", "Richardson", email = "sylvia.richardson@mrc-bsu.cam.ac.uk", role = c("ctb")),
             person("Waldir", "Leoncio", email = "w.l.netto@medisin.uio.no", role = c("ctb")),
             person("Alex", "Lewin", email = "alex.lewin@lshtm.ac.uk", role = c("aut")),
             person("Manuela", "Zucknick", email = "manuela.zucknick@medisin.uio.no", role = c("aut")))
Description: Bayesian seemingly unrelated regression with general variable selection and dense/sparse covariance matrix. The sparse seemingly unrelated regression is described in Bottolo et al. (2021) <doi:10.1111/rssc.12490>, the software paper is in Zhao et al. (2021) <doi:10.18637/jss.v100.i11>, and the model with random effects is described in Zhao et al. (2024) <doi:10.1093/jrsssc/qlad102>.
License: MIT + file LICENSE
Copyright: The C++ files pugixml.cpp, pugixml.hpp and pugiconfig.hpp
        are used with Copyright (C) 2006-2018 from Arseny Kapoulkine
        and Copyright (C) 2003 from Kristen Wegner. The R function
        vertical.image.legend() is used with Copyright (C) 2013 from
        Jenise Swall.
VignetteBuilder: R.rsp, knitr, rmarkdown
Depends: R (>= 3.5.0)
Encoding: UTF-8
LinkingTo: Rcpp, RcppArmadillo
Imports: Rcpp, igraph, Matrix, tikzDevice, stats, utils, grDevices,
        graphics
Suggests: R.rsp, knitr, rmarkdown, BDgraph, data.table, plyr, scrime
LazyData: true
NeedsCompilation: yes
SystemRequirements: C++17
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-17 18:28:12 UTC; zhiz
Author: Marco Banterle [aut],
  Zhi Zhao [aut, cre],
  Leonardo Bottolo [ctb],
  Sylvia Richardson [ctb],
  Waldir Leoncio [ctb],
  Alex Lewin [aut],
  Manuela Zucknick [aut]
Maintainer: Zhi Zhao <zhi.zhao@medisin.uio.no>
Repository: CRAN
Date/Publication: 2026-05-27 06:30:09 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-27 07:38:31 UTC; unix
Archs: BayesSUR.so.dSYM
