# Generated by roxygen2: do not edit by hand

S3method(coef,BayesSUR)
S3method(fitted,BayesSUR)
S3method(plot,BayesSUR)
S3method(predict,BayesSUR)
S3method(print,BayesSUR)
S3method(summary,BayesSUR)
export(BayesSUR)
export(elpd)
export(getEstimator)
export(plotCPO)
export(plotEstimator)
export(plotGraph)
export(plotMCMCdiag)
export(plotManhattan)
export(plotNetwork)
importFrom(Matrix,Matrix)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.flush)
importFrom(grDevices,dev.hold)
importFrom(grDevices,dev.off)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,gray)
importFrom(grDevices,grey)
importFrom(grDevices,hcl.colors)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,image)
importFrom(graphics,legend)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.default)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertices)
importFrom(igraph,ecount)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,gsize)
importFrom(igraph,layout_in_circle)
importFrom(igraph,layout_with_fr)
importFrom(igraph,plot.igraph)
importFrom(parallel,detectCores)
importFrom(stats,density)
importFrom(stats,sd)
importFrom(tikzDevice,tikz)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(BayesSUR)
